@@ -18,6 +18,7 @@
 import datetime
 import hashlib
 import httplib
+import logging
 import os
 import json
 import unittest
@@ -27,12 +28,14 @@ import webob
 
 from glance.api import v1 as server
 from glance.common import context
+from glance.common import utils
 from glance.registry import context as rcontext
-from glance.registry import server as rserver
+from glance.registry.api import v1 as rserver
 from glance.registry.db import api as db_api
 from glance.registry.db import models as db_models
 from glance.tests import stubs
 
+
 OPTIONS = {'sql_connection': 'sqlite://',
            'verbose': False,
            'debug': False,
@@ -43,6 +46,50 @@ OPTIONS = {'sql_connection': 'sqlite://',
            'context_class': 'glance.registry.context.RequestContext'}
 
 
+class TestRegistryDb(unittest.TestCase):
+
+    def setUp(self):
+        """Establish a clean test environment"""
+        self.stubs = stubout.StubOutForTesting()
+
+    def test_bad_sql_connection(self):
+        """
+        Test that a bad sql_connection option supplied to the registry
+        API controller results in a) an Exception being thrown and b)
+        a message being logged to the registry log file...
+        """
+        bad_options = {'verbose': True,
+                       'debug': True,
+                       'sql_connection': 'baddriver:///'}
+        # We set this to None to trigger a reconfigure, otherwise
+        # other modules may have already correctly configured the DB
+        orig_engine = db_api._ENGINE
+        db_api._ENGINE = None
+        self.assertRaises(ImportError, db_api.configure_db,
+                          bad_options)
+        exc_raised = False
+        self.log_written = False
+
+        def fake_log_error(msg):
+            if 'Error configuring registry database' in msg:
+                self.log_written = True
+
+        self.stubs.Set(db_api.logger, 'error', fake_log_error)
+        try:
+            api_obj = rserver.API(bad_options)
+        except ImportError:
+            exc_raised = True
+        finally:
+            db_api._ENGINE = orig_engine
+
+        self.assertTrue(exc_raised)
+        self.assertTrue(self.log_written)
+
+    def tearDown(self):
+        """Clear the test environment"""
+        self.stubs.UnsetAll()
+
+
 class TestRegistryAPI(unittest.TestCase):
     def setUp(self):
         """Establish a clean test environment"""
@@ -1208,6 +1255,96 @@ class TestRegistryAPI(unittest.TestCase):
         for image in images:
             self.assertTrue(image['size'] <= 19 and image['size'] >= 18)
 
+    def test_get_details_filter_changes_since(self):
+        """
+        Tests that the /images/detail registry API returns list of
+        public images that have a size less than or equal to size_max
+        """
+        dt1 = datetime.datetime.utcnow() - datetime.timedelta(1)
+        iso1 = utils.isotime(dt1)
+
+        dt2 = datetime.datetime.utcnow() + datetime.timedelta(1)
+        iso2 = utils.isotime(dt2)
+
+        dt3 = datetime.datetime.utcnow() + datetime.timedelta(2)
+        iso3 = utils.isotime(dt3)
+
+        dt4 = datetime.datetime.utcnow() + datetime.timedelta(3)
+        iso4 = utils.isotime(dt4)
+
+        extra_fixture = {'id': 3,
+                         'status': 'active',
+                         'is_public': True,
+                         'disk_format': 'vhd',
+                         'container_format': 'ovf',
+                         'name': 'fake image #3',
+                         'size': 18,
+                         'checksum': None}
+
+        db_api.image_create(self.context, extra_fixture)
+        db_api.image_destroy(self.context, 3)
+
+        extra_fixture = {'id': 4,
+                         'status': 'active',
+                         'is_public': True,
+                         'disk_format': 'ami',
+                         'container_format': 'ami',
+                         'name': 'fake image #4',
+                         'size': 20,
+                         'checksum': None,
+                         'created_at': dt3,
+                         'updated_at': dt3}
+
+        db_api.image_create(self.context, extra_fixture)
+
+        # Check a standard list, 4 images in db (2 deleted)
+        req = webob.Request.blank('/images/detail')
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 2)
+        self.assertEqual(images[0]['id'], 4)
+        self.assertEqual(images[1]['id'], 2)
+
+        # Expect 3 images (1 deleted)
+        req = webob.Request.blank('/images/detail?changes-since=%s' % iso1)
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 3)
+        self.assertEqual(images[0]['id'], 4)
+        self.assertEqual(images[1]['id'], 3)  # deleted
+        self.assertEqual(images[2]['id'], 2)
+
+        # Expect 1 images (0 deleted)
+        req = webob.Request.blank('/images/detail?changes-since=%s' % iso2)
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 1)
+        self.assertEqual(images[0]['id'], 4)
+
+        # Expect 0 images (0 deleted)
+        req = webob.Request.blank('/images/detail?changes-since=%s' % iso4)
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 0)
+
+        # Bad request (empty changes-since param)
+        req = webob.Request.blank('/images/detail?changes-since=')
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 400)
+
+        # Bad request (invalid changes-since param)
+        req = webob.Request.blank('/images/detail?changes-since=2011-09-05')
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 400)
+
     def test_get_details_filter_property(self):
         """
         Tests that the /images/detail registry API returns list of
@@ -1930,10 +2067,6 @@ class TestGlanceAPI(unittest.TestCase):
         res = req.get_response(self.api)
         self.assertEquals(res.status_int, httplib.CREATED)
 
-        res_body = json.loads(res.body)['image']
-        self.assertEquals(res_body['location'],
-                          'file:///tmp/glance-tests/3')
-
         # Test that the Location: header is set to the URI to
         # edit the newly-created image, as required by APP.
         # See LP Bug #719825
@@ -1981,6 +2114,149 @@ class TestGlanceAPI(unittest.TestCase):
         self.assertEquals(int(images[1]['id']), 2)
         self.assertEquals(int(images[2]['id']), 4)
 
+    def test_get_details_filter_changes_since(self):
+        """
+        Tests that the /images/detail registry API returns list of
+        public images that have a size less than or equal to size_max
+        """
+        dt1 = datetime.datetime.utcnow() - datetime.timedelta(1)
+        iso1 = utils.isotime(dt1)
+
+        dt2 = datetime.datetime.utcnow() + datetime.timedelta(1)
+        iso2 = utils.isotime(dt2)
+
+        dt3 = datetime.datetime.utcnow() + datetime.timedelta(2)
+        iso3 = utils.isotime(dt3)
+
+        dt4 = datetime.datetime.utcnow() + datetime.timedelta(3)
+        iso4 = utils.isotime(dt4)
+
+        extra_fixture = {'id': 3,
+                         'status': 'active',
+                         'is_public': True,
+                         'disk_format': 'vhd',
+                         'container_format': 'ovf',
+                         'name': 'fake image #3',
+                         'size': 18,
+                         'checksum': None}
+
+        db_api.image_create(self.context, extra_fixture)
+        db_api.image_destroy(self.context, 3)
+
+        extra_fixture = {'id': 4,
+                         'status': 'active',
+                         'is_public': True,
+                         'disk_format': 'ami',
+                         'container_format': 'ami',
+                         'name': 'fake image #4',
+                         'size': 20,
+                         'checksum': None,
+                         'created_at': dt3,
+                         'updated_at': dt3}
+
+        db_api.image_create(self.context, extra_fixture)
+
+        # Check a standard list, 4 images in db (2 deleted)
+        req = webob.Request.blank('/images/detail')
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 2)
+        self.assertEqual(images[0]['id'], 4)
+        self.assertEqual(images[1]['id'], 2)
+
+        # Expect 3 images (1 deleted)
+        req = webob.Request.blank('/images/detail?changes-since=%s' % iso1)
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 3)
+        self.assertEqual(images[0]['id'], 4)
+        self.assertEqual(images[1]['id'], 3)  # deleted
+        self.assertEqual(images[2]['id'], 2)
+
+        # Expect 1 images (0 deleted)
+        req = webob.Request.blank('/images/detail?changes-since=%s' % iso2)
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 1)
+        self.assertEqual(images[0]['id'], 4)
+
+        # Expect 0 images (0 deleted)
+        req = webob.Request.blank('/images/detail?changes-since=%s' % iso4)
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 200)
+        res_dict = json.loads(res.body)
+        images = res_dict['images']
+        self.assertEquals(len(images), 0)
+
+        # Bad request (empty changes-since param)
+        req = webob.Request.blank('/images/detail?changes-since=')
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 400)
+
+        # Bad request (invalid changes-since param)
+        req = webob.Request.blank('/images/detail?changes-since=2011-09-05')
+        res = req.get_response(self.api)
+        self.assertEquals(res.status_int, 400)
+
+    def test_store_location_not_revealed(self):
+        """
+        Test that the internal store location is NOT revealed
+        through the API server
+        """
+        # Check index and details...
+        for url in ('/images', '/images/detail'):
+            req = webob.Request.blank(url)
+            res = req.get_response(self.api)
+            self.assertEquals(res.status_int, 200)
+            res_dict = json.loads(res.body)
+
+            images = res_dict['images']
+            num_locations = sum([1 for record in images
+                                if 'location' in record.keys()])
+            self.assertEquals(0, num_locations, images)
+
+        # Check GET
+        req = webob.Request.blank("/images/2")
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        self.assertFalse('X-Image-Meta-Location' in res.headers)
+
+        # Check HEAD
+        req = webob.Request.blank("/images/2")
+        req.method = 'HEAD'
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        self.assertFalse('X-Image-Meta-Location' in res.headers)
+
+        # Check PUT
+        req = webob.Request.blank("/images/2")
+        req.body = res.body
+        req.method = 'PUT'
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        res_body = json.loads(res.body)
+        self.assertFalse('location' in res_body['image'])
+
+        # Check POST
+        req = webob.Request.blank("/images")
+        headers = {'x-image-meta-location': 'http://localhost',
+                   'x-image-meta-disk-format': 'vhd',
+                   'x-image-meta-container-format': 'ovf',
+                   'x-image-meta-name': 'fake image #3'}
+        for k, v in headers.iteritems():
+            req.headers[k] = v
+        req.method = 'POST'
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 201)
+        res_body = json.loads(res.body)
+        self.assertFalse('location' in res_body['image'])
+
     def test_image_is_checksummed(self):
         """Test that the image contents are checksummed properly"""
         fixture_headers = {'x-image-meta-store': 'file',
@@ -2001,8 +2277,6 @@ class TestGlanceAPI(unittest.TestCase):
         self.assertEquals(res.status_int, httplib.CREATED)
 
         res_body = json.loads(res.body)['image']
-        self.assertEquals(res_body['location'],
-                          'file:///tmp/glance-tests/3')
         self.assertEquals(image_checksum, res_body['checksum'],
                           "Mismatched checksum. Expected %s, got %s" %
                           (image_checksum, res_body['checksum']))
@@ -2121,9 +2395,8 @@ class TestGlanceAPI(unittest.TestCase):
         that had had its save process killed manually results in failure
         because the location attribute is None.
         """
-        # Add an image the way that glance-upload adds an image...
-        # by reserving a place in the database for an image without
-        # really any attributes or information on the image and then
+        # Add an image by reserving a place in the database for an image
+        # without really any attributes or information on the image and then
         # later doing an update with the image body and other attributes.
         # We will stop the process after the reservation stage, then
         # try to delete the image.
